﻿using System;
using System.Linq;
using System.Windows.Forms;
using DicomObjects;
using System.IO;
using DicomObjects.Enums;

namespace SR_HTML_Render
{
    public partial class Form1 : Form
    {
        private const string PatientName = "Patient Name";
        private const string PatientId = "Patient ID";
        private const string PatientDateofBirth = "Patient Date of Birth";
        private const string PatientSex = "Patient Sex";
        private const string CompleteFlag = "Complete Flag";
        private const string VerificationFlag = "Verification Flag";
        private const string ValueType_Container = "CONTAINER";
        private const string ValueType_Code = "CODE";
        private const string ValueType_Text = "TEXT";
        private const string ValueType_Image = "IMAGE";
        private const string ValueType_DateTime = "DateTime";
        private const string ValueType_Date = "Date";
        private const string ValueType_Time = "Time";
        private const string ValueType_Pname = "PNAME";
        private const string ValueType_Number = "NUM";
        private const string ReadSRFile_ErrorMessage = "This is not SR file. Please choose a SR file.";

        DicomDataSet SR;
        public Form1()
        {
            InitializeComponent();
        }

        readonly string[] SequencesToRead = { "Measurement Group" };
        private void Load_Dicom_SR_Click(object sender, EventArgs e)
        {
            var samplePath = Path.Combine(SampleExtensions.GetSampleImageFolderPath(), SampleExtensions.NON_IMAGE);
            OpenFileDialog ofd = new OpenFileDialog();
            if (Directory.Exists(samplePath))
                ofd.InitialDirectory = samplePath;
            ofd.ShowDialog();

            if (!string.IsNullOrEmpty(ofd.FileName))
            {
                progressBar1.Visible = true;
                SR = new DicomDataSet(ofd.FileName);
                string content = ReadSRContent(SR);
                WriteToTempFile(content);
                webBrowser1.Navigate(Application.StartupPath + "\\temp.html");
                progressBar1.Visible = false;
            }
        }

        private void WriteToTempFile(string content)
        {
            string fileName = Path.Combine(Application.StartupPath,"temp.html");
            File.WriteAllText(fileName, content);
        }

        private string ReadSRContent(DicomDataSet SR)
        {
            string result, title;
            title = ReadSRTitle(SR);
            result = "<center><h2><b><u>DicomObjects Sample SR</b></u></h2></center>";
            result = $"{result} <br>";
            result = $"{result} <center><h2><font color = blue>{title}</font></h2></center>";
            result = $"{result} <hr>";
            result = $"{result} <br>";

            // =====================  Basic Patient Information  =====================
            // Patient Name            
            string patientName = SR[Keyword.PatientName].ExistsWithValue ? SR[Keyword.PatientName].Value.ToString() : "";
            AddBasicTag(ref result, PatientName, patientName);

            // Patient ID            
            string patientID = SR[Keyword.PatientID].ExistsWithValue ? SR[Keyword.PatientID].Value.ToString() : "";
            AddBasicTag(ref result, PatientId, patientID);

            // Date of Birth
            string patientDOB = SR[Keyword.PatientBirthDate].ExistsWithValue ? SR[Keyword.PatientBirthDate].Value.ToString() : "";
            AddBasicTag(ref result, PatientDateofBirth, patientDOB);

            // Sex
            string patientSex = SR[Keyword.PatientSex].ExistsWithValue ? SR[Keyword.PatientSex].Value.ToString() : "";
            AddBasicTag(ref result, PatientSex, patientSex);

            // Completion Flag
            string completeFlag = SR[Keyword.CompletionFlag].ExistsWithValue ? SR[Keyword.CompletionFlag].Value.ToString() : "";
            AddBasicTag(ref result, CompleteFlag, completeFlag);

            // Verification Flag
            string verificationFlag = SR[Keyword.VerificationFlag].ExistsWithValue ? SR[Keyword.VerificationFlag].Value.ToString() : "";
            AddBasicTag(ref result, VerificationFlag, verificationFlag);

            result = $"{result} <br><hr><br>";

            //===============  Get Entire Content Sequence  ===============
            DicomDataSetCollection contentSequence = (DicomDataSetCollection)SR[Keyword.ContentSequence].Value;
            foreach (DicomDataSet contentSQItem in contentSequence)
            {
                // Read Each Individual Sequence Item
                ReadSRContentItem(contentSQItem, ref result, 1, false);     // false to not read sequences by default
            }
            return (result);
        }

        private void ReadSRContentItem(DicomDataSet SR, ref string result, int level, bool ReadSequence)
        {
            bool readSQ = ReadSequence;
            // Read Container Item
            if (SR[Keyword.ValueType].ExistsWithValue)
            {
                string valueType = SR[Keyword.ValueType].Value.ToString();
                if (valueType == ValueType_Container || readSQ)
                {
                    DicomDataSetCollection conceptNameCodeSQ = (DicomDataSetCollection)SR[Keyword.ConceptNameCodeSequence].Value;
                    string codeMeaning = conceptNameCodeSQ[0][Keyword.CodeMeaning].Value + "";

                    if (SequencesToRead.Contains(codeMeaning))
                        readSQ = true;                              //  Identifying sequence to be read

                    if (!readSQ)                                     // avoid repeating cName to output html
                        AddContainerTag(ref result, codeMeaning, level);

                    if (SR[Keyword.ContentSequence].ExistsWithValue)
                    {
                        level++;
                        DicomDataSetCollection contentSQ = (DicomDataSetCollection)SR[Keyword.ContentSequence].Value;
                        foreach (DicomDataSet contentSQItem in contentSQ)
                        {
                            ReadSRContentItem(contentSQItem, ref result, level, readSQ);
                        }
                    }
                }

                if (valueType == ValueType_Code)
                {
                    DicomDataSetCollection ds1 = (DicomDataSetCollection)SR[Keyword.ConceptNameCodeSequence].Value;
                    string codeName = ds1[0][Keyword.CodeMeaning].Value.ToString();
                    DicomDataSetCollection ds2 = (DicomDataSetCollection)SR[Keyword.ConceptCodeSequence].Value;
                    string codeValue = ds2[0][Keyword.CodeValue].Value.ToString();
                    string codingSchemeDesignator = ds2[0][Keyword.CodingSchemeDesignator].Value.ToString();
                    string codeMeaning = ds2[0][Keyword.CodeMeaning].Value.ToString();
                    AddContainerTag(ref result, codeName + " : " + codeMeaning + " (" + codeValue + "," + codingSchemeDesignator + ")", level);
                }

                if (valueType == ValueType_Text)
                {
                    DicomDataSetCollection dx2 = (DicomDataSetCollection)SR[Keyword.ConceptNameCodeSequence].Value;
                    string codeMean = dx2[0][Keyword.CodeMeaning].Value.ToString();
                    codeMean = $"<b><u>{codeMean}</u></b>";
                    string textValue = SR[Keyword.TextValue].Value + "";
                    AddContainerTag(ref result, codeMean + " : " + textValue, level);
                }

                // read Object (Image, Waveform,etc. Composite Object) Item
                if (valueType == ValueType_Image)
                {
                    DicomDataSetCollection refds = (DicomDataSetCollection)SR[Keyword.ReferencedSOPSequence].Value;
                    string refInsUI = (string)refds[0][0x0008, 0x1155].Value + "";
                    DicomDataSetCollection ids = (DicomDataSetCollection)SR[Keyword.ConceptNameCodeSequence].Value;
                    string codeMeaning = (string)ids[0][Keyword.CodeMeaning].Value + "";
                    codeMeaning = $"<b><u>{codeMeaning}</u></b>";
                    AddContainerTag(ref result, codeMeaning + " : " + refInsUI, level);
                }

                // read Other Text Format Item
                if (valueType == ValueType_DateTime || valueType == ValueType_Date ||
                     valueType == ValueType_Time || valueType == ValueType_Pname)
                {
                    DicomDataSetCollection ds3 = (DicomDataSetCollection)SR[Keyword.ConceptNameCodeSequence].Value;
                    string codeName = ds3[0][Keyword.CodeMeaning].Value + "";
                    string codeMeaning = SR[Keyword.PersonName].Value + "";
                    AddContainerTag(ref result, codeName + " : " + codeMeaning, level);
                }

                // read Num Format Item
                if (valueType == ValueType_Number)
                {
                    DicomDataSetCollection ds1 = (DicomDataSetCollection)SR[Keyword.ConceptNameCodeSequence].Value;                                                  // Concept Name Code Sequence           
                    string codeName = (string)ds1[0][Keyword.CodeMeaning].Value;
                    DicomDataSetCollection ds2 = (DicomDataSetCollection)SR[Keyword.MeasuredValueSequence].Value;
                    DicomDataSetCollection MeasurementUnitsCodeSQ = (DicomDataSetCollection)ds2[0][Keyword.MeasurementUnitsCodeSequence].Value;                            // Concept Code Sequence
                    if (MeasurementUnitsCodeSQ != null)
                    {
                        string value = string.Join(", ", (string[])ds2[0][Keyword.NumericValue].Value);
                        string codeName2 = (string)MeasurementUnitsCodeSQ[0][Keyword.CodeMeaning].Value;
                        AddContainerTag(ref result, codeName + " : " + value + codeName2, level);
                    }
                }
            }
        }


        private void AddContainerTag(ref string s, string addin, int level)
        {
            switch (level)
            {
                case 1:
                    s = $"{s}<font size = 3 face = Arial color = gray><b>{addin}</b></font><br>";
                    break;
                default:
                    s = $"{s}<ul><p><font size = 2 face = Arial><b>{addin}</b></font></p></ul>";
                    break;
                //case 3:
                //    s = s + "<ul><ul><font size = 2 face = Arial>" + "<b>" + addin + "</b>" + "</font></ul></ul>";
                //    s = s + "<ul><ul><font size = 2 face = Arial>" + "<b>" + addin + "</b>" + "</font></ul></ul>";
                //    break;
                //case 4:
                //    s = s + "<ul><ul><ul><font size = 2 face = Arial>" + "<b>" + addin + "</b>" + "</font></ul></ul></ul>";
                //    break;

                //case 5:
                //    s = s + "<ul><ul><ul><ul><font size = 2 face = Arial>" + "<b>" + addin + "</b>" + "</font></ul></ul></ul></ul>";
                //    s = s + "<ul><ul><ul><ul><ul><font size = 2 face = Arial>" + "<b>" + addin + "</b>" + "</font></ul></ul></ul></ul></ul>";
                //    break;
            }
        }

        void AddBasicTag(ref string s, string title, string addin)
        {
            s = $"{s}<font size = 2 face = Arial>{title}: {addin}</font><br>";
        }

        string ReadSRTitle(DicomDataSet SR)
        {
            DicomDataSetCollection dsc;
            try
            {
                dsc = (DicomDataSetCollection)SR[Keyword.ConceptNameCodeSequence].Value;
                if (dsc != null)
                    return ((string)dsc[0][Keyword.CodeMeaning].Value + " ");
                else MessageBox.Show(ReadSRFile_ErrorMessage);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }

            return string.Empty;
        }
    }
}
